@@ -20,10 +20,15 @@ date_names <- function(
   day_ab = day,
   am_pm = c("AM", "PM")
 ) {
-  stopifnot(is.character(mon), length(mon) == 12)
-  stopifnot(is.character(mon_ab), length(mon_ab) == 12)
-  stopifnot(is.character(day), length(day) == 7)
-  stopifnot(is.character(day_ab), length(day_ab) == 7)
+  check_character(mon)
+  check_character(mon_ab)
+  check_character(day)
+  check_character(day_ab)
+
+  check_length(mon, 12)
+  check_length(mon_ab, 12)
+  check_length(day, 7)
+  check_length(day_ab, 7)
 
   structure(
     list(
@@ -42,15 +47,20 @@ date_names <- function(
 #' @param language A BCP 47 locale, made up of a language and a region,
 #'   e.g. `"en_US"` for American English. See `date_names_langs()`
 #'   for a complete list of available locales.
-date_names_lang <- function(language) {
-  stopifnot(is.character(language), length(language) == 1)
+#' @inheritParams rlang::args_error_context
+date_names_lang <- function(language, call = caller_env()) {
+  check_string(
+    language,
+    arg = caller_arg(language),
+    call = call
+  )
 
-  symbols <- date_symbols[[language]]
-  if (is.null(symbols)) {
-    stop("Unknown language '", language, "'", call. = FALSE)
+  symbol <- date_symbols[[language]]
+  if (is.null(symbol)) {
+    cli::cli_abort("Unrecognized language code: {.val {language}}", call = call)
   }
 
-  symbols
+  symbol
 }
 
 #' @export
@@ -60,6 +70,7 @@ date_names_langs <- function() {
 }
 
 # Conditionally exported in zzz.R
+#' @noRd
 # @export
 print.date_names <- function(x, ...) {
   cat("<date_names>\n")
