@@ -47,7 +47,11 @@ locale <- function(
   if (is.character(date_names)) {
     date_names <- date_names_lang(date_names)
   }
-  stopifnot(is.date_names(date_names))
+  if (!is.date_names(date_names)) {
+    cli::cli_abort(
+      "{.arg date_names} must be a language code like {.val en} or an object created by {.fun date_names}."
+    )
+  }
 
   if (missing(grouping_mark) && !missing(decimal_mark)) {
     grouping_mark <- if (decimal_mark == ".") "," else "."
@@ -55,10 +59,15 @@ locale <- function(
     decimal_mark <- if (grouping_mark == ".") "," else "."
   }
 
-  stopifnot(is.character(decimal_mark), length(decimal_mark) == 1)
-  stopifnot(is.character(grouping_mark), length(grouping_mark) == 1)
+  check_string(decimal_mark)
+  check_string(grouping_mark)
   if (decimal_mark == grouping_mark) {
-    stop("`decimal_mark` and `grouping_mark` must be different", call. = FALSE)
+    cli::cli_abort(
+      c(
+        "{.arg decimal_mark} and {.arg grouping_mark} must be different.",
+        "i" = "Both were specified as {.val {decimal_mark}}."
+      )
+    )
   }
 
   tz <- check_tz(tz)
@@ -81,6 +90,7 @@ locale <- function(
 is.locale <- function(x) inherits(x, "locale")
 
 # Conditionally exported in zzz.R
+#' @noRd
 # @export
 print.locale <- function(x, ...) {
   cat("<locale>\n")
@@ -113,11 +123,14 @@ default_locale <- function() {
   loc
 }
 
-check_tz <- function(x) {
-  stopifnot(is.character(x), length(x) == 1)
+check_tz <- function(x, call = caller_env()) {
+  check_string(x, arg = caller_arg(x), call = call)
+
+  tz_source <- ""
 
   if (identical(x, "")) {
     x <- Sys.timezone()
+    tz_source <- "system "
 
     if (identical(x, "") || identical(x, NA_character_)) {
       x <- "UTC"
@@ -127,14 +140,14 @@ check_tz <- function(x) {
   if (x %in% tzdb::tzdb_names()) {
     x
   } else {
-    stop("Unknown TZ ", x, call. = FALSE)
+    cli::cli_abort("Unknown {tz_source}timezone: {.val {x}}.", call = call)
   }
 }
 
-# see https://github.com/tidyverse/readr/pull/1537 for why this more relaxed
+# see https://github.com/tidyverse/readr/pull/1537 for why this is more relaxed
 # than you might expect (and than it used to be)
-check_encoding <- function(x) {
-  stopifnot(is.character(x), length(x) == 1)
+check_encoding <- function(x, call = caller_env()) {
+  check_string(x, arg = caller_arg(x), call = call)
 
   # portable encoding names
   if (x %in% c("latin1", "UTF-8")) {
@@ -144,15 +157,10 @@ check_encoding <- function(x) {
   # 'iconvlist' could be incomplete (musl) or even unavailable
   known <- tryCatch(iconvlist(), error = identity)
   if (inherits(known, "error")) {
-    warning("Could not check `encoding` against `iconvlist()`.", call. = FALSE)
+    cli::cli_warn("Could not check {.arg encoding} against {.fun iconvlist}.")
   } else if (tolower(x) %in% tolower(known)) {
     TRUE
   } else {
-    warning(
-      "Unknown encoding ",
-      encodeString(x, quote = '"'),
-      ".",
-      call. = FALSE
-    )
+    cli::cli_warn("{.arg encoding} not found in {.fun iconvlist}: {.val {x}}.")
   }
 }
