@@ -1,3 +1,14 @@
+vroom_tempfile <- function(fileext = "", pattern = "vroom-") {
+  dir <- Sys.getenv("VROOM_TEMP_PATH")
+  if (!nzchar(dir)) {
+    dir <- tempdir()
+  }
+  if (nzchar(fileext) && !startsWith(fileext, ".")) {
+    fileext <- paste0(".", fileext)
+  }
+  tempfile(pattern = pattern, tmpdir = dir, fileext = fileext)
+}
+
 is_ascii_compatible <- function(encoding) {
   identical(
     iconv(list(charToRaw("\n")), from = "ASCII", to = encoding, toRaw = TRUE)[[
@@ -8,26 +19,26 @@ is_ascii_compatible <- function(encoding) {
 }
 
 # this is about the encoding of the file (contents), not the filepath
-reencode_file <- function(path, encoding) {
-  if (length(path) > 1) {
-    stop(
-      sprintf(
-        "Reading files of encoding '%s' can only be done for single files at a time",
-        encoding
+reencode_file <- function(file, encoding, call = caller_env()) {
+  if (length(file) > 1) {
+    cli::cli_abort(
+      c(
+        "!" = "Encoding {.val {encoding}} is only supported when reading a single input.",
+        "i" = "{.arg file} has length {length(file)}."
       ),
-      call. = FALSE
+      call = call
     )
   }
 
-  if (inherits(path[[1]], "connection")) {
-    in_con <- path[[1]]
+  if (inherits(file[[1]], "connection")) {
+    in_con <- file[[1]]
   } else {
-    in_con <- file(path[[1]])
+    in_con <- file(file[[1]])
   }
-  out_file <- tempfile()
+  out_file <- vroom_tempfile(pattern = "vroom-reencode-file-")
   out_con <- file(out_file)
   convert_connection(in_con, out_con, encoding, "UTF-8")
-  withr::defer(unlink(out_file), envir = parent.frame())
+  withr::defer(unlink(out_file), envir = call)
   return(list(out_file))
 }
 
@@ -66,10 +77,11 @@ standardise_path <- function(
   if (!is.character(path)) {
     cli::cli_abort(
       c(
-        "{.arg {arg}} is not one of the supported inputs:",
+        "{.arg {arg}} must be one of the supported input types:",
         "*" = "A filepath or character vector of filepaths",
         "*" = "A connection or list of connections",
-        "*" = "Literal or raw input"
+        "*" = "Literal or raw input",
+        "x" = "{.arg {arg}} is {obj_type_friendly(path)}."
       ),
       call = call
     )
@@ -79,11 +91,11 @@ standardise_path <- function(
     if (length(path) > 1) {
       path <- paste(path, collapse = "\n")
     }
-    return(list(chr_to_file(path, envir = parent.frame())))
+    return(list(chr_to_file(path, call = call)))
   }
 
   if (any(grepl("\n", path))) {
-    lifecycle::deprecate_soft(
+    lifecycle::deprecate_warn(
       "1.5.0",
       "vroom(file = 'must use `I()` for literal data')",
       details = c(
@@ -96,7 +108,38 @@ standardise_path <- function(
       ),
       user_env = user_env
     )
-    return(list(chr_to_file(path, envir = parent.frame())))
+    return(list(chr_to_file(path, call = call)))
+  }
+
+  # Handle remote compressed files from R, before heading to C++, so we can
+  # associate cleanup with the desired environment.
+  is_remote_compressed <- vapply(
+    path,
+    function(p) {
+      is_url(p) && tolower(tools::file_ext(p)) %in% c("gz", "bz2", "xz", "zip")
+    },
+    logical(1)
+  )
+  if (all(is_remote_compressed)) {
+    return(lapply(path, function(p) {
+      ext <- tolower(tools::file_ext(p))
+      local_path <- download_file(p, ext, call = call)
+      withr::defer(unlink(local_path), envir = call)
+      switch(
+        ext,
+        gz = gzfile(local_path),
+        bz2 = bzfile(local_path),
+        xz = xzfile(local_path),
+        zip = zipfile(local_path)
+      )
+    }))
+  }
+
+  if (any(is_remote_compressed)) {
+    cli::cli_abort(
+      "{.arg {arg}} cannot be a mix of remote compressed and other inputs.",
+      call = call
+    )
   }
 
   as.list(enc2utf8(path))
@@ -113,7 +156,7 @@ standardise_connection <- function(con) {
   con
 }
 
-standardise_one_path <- function(path, write = FALSE) {
+connection_or_filepath <- function(path, write = FALSE, call = caller_env()) {
   if (is.raw(path)) {
     return(rawConnection(path, "rb"))
   }
@@ -123,73 +166,64 @@ standardise_one_path <- function(path, write = FALSE) {
   }
 
   if (is_url(path)) {
+    ext <- tolower(tools::file_ext(path))
+    if (ext %in% c("gz", "bz2", "xz", "zip")) {
+      cli::cli_abort(
+        "Remote compressed files must be handled upstream of this function.",
+        .internal = TRUE
+      )
+    }
     if (requireNamespace("curl", quietly = TRUE)) {
       con <- curl::curl(path)
     } else {
       inform("`curl` package not installed, falling back to using `url()`")
       con <- url(path)
     }
-    ext <- tolower(tools::file_ext(path))
-    return(
-      switch(
-        ext,
-        zip = ,
-        bz2 = ,
-        xz = {
-          close(con)
-          stop(
-            "Reading from remote `",
-            ext,
-            "` compressed files is not supported,\n",
-            "  download the files locally first.",
-            call. = FALSE
-          )
-        },
-        gz = gzcon(con),
-        con
-      )
-    )
+    return(con)
   }
 
   path <- enc2utf8(path)
 
-  p <- split_path_ext(basename_utf8(path))
-
   if (write) {
     path <- normalizePath_utf8(path, mustWork = FALSE)
   } else {
     path <- check_path(path)
   }
 
-  if (is_installed("archive")) {
-    formats <- archive_formats(p$extension)
-    extension <- p$extension
-    while (is.null(formats) && nzchar(extension)) {
-      extension <- split_path_ext(extension)$extension
-      formats <- archive_formats(extension)
-    }
-    if (!is.null(formats)) {
-      p$extension <- extension
-      if (write) {
-        if (is.null(formats[[1]])) {
-          return(archive::file_write(path, filter = formats[[2]]))
-        }
-        return(archive::archive_write(
-          path,
-          p$path,
-          format = formats[[1]],
-          filter = formats[[2]]
-        ))
-      }
+  p <- split_path_ext(basename_utf8(path))
+  extension <- p$extension
+  formats <- archive_formats(extension)
+  while (is.null(formats) && nzchar(extension)) {
+    extension <- split_path_ext(extension)$extension
+    formats <- archive_formats(extension)
+  }
+  needs_archive <- !is.null(formats) && (write || extension != "zip")
+
+  if (needs_archive) {
+    reason <- glue(
+      "to {if (write) 'write' else 'read'} `.{p$extension}` files."
+    )
+    rlang::check_installed("archive", reason = reason, call = call)
+
+    if (write) {
       if (is.null(formats[[1]])) {
-        return(archive::file_read(path, filter = formats[[2]]))
+        return(archive::file_write(path, filter = formats[[2]]))
       }
-      return(archive::archive_read(
+      return(archive::archive_write(
         path,
+        p$path,
         format = formats[[1]],
         filter = formats[[2]]
       ))
     }
+    if (is.null(formats[[1]])) {
+      return(archive::file_read(path, filter = formats[[2]]))
+    }
+    return(archive::archive_read(
+      path,
+      format = formats[[1]],
+      filter = formats[[2]]
+    ))
   }
 
   if (!write) {
@@ -203,15 +237,21 @@ standardise_one_path <- function(path, write = FALSE) {
   }
 
   if (write && compression == "zip") {
-    stop("Can only read from, not write to, .zip", call. = FALSE)
+    cli::cli_abort(
+      c(
+        "Can only read from, not write to, {.val .zip} files.",
+        "i" = "Install the {.pkg archive} package to write {.val .zip} files."
+      ),
+      call = call
+    )
   }
 
   switch(
     compression,
-    gz = gzfile(path, ""),
-    bz2 = bzfile(path, ""),
-    xz = xzfile(path, ""),
-    zip = zipfile(path, ""),
+    gz = gzfile(path),
+    bz2 = bzfile(path),
+    xz = xzfile(path),
+    zip = zipfile(path),
     if (!has_trailing_newline(path)) {
       file(path)
     } else {
@@ -220,6 +260,23 @@ standardise_one_path <- function(path, write = FALSE) {
   )
 }
 
+# Safe wrapper around base::open() that ensures connections are cleaned up
+# even when open() fails. We need this especially for opening connections from
+# C++. If base::open() fails in that context, R will long jump and there aren't
+# good options for arranging the necessary cleanup.
+open_safely <- function(con, open_mode = "rb") {
+  tryCatch(
+    open(con, open_mode),
+    error = function(e) {
+      # The failed connection is already "closed", so this attempt to close() it
+      # is really about removing it from R's connection table.
+      try(close(con), silent = TRUE)
+
+      stop(conditionMessage(e), call. = FALSE)
+    }
+  )
+}
+
 split_path_ext <- function(path) {
   regex <- "^([^.]*)[.](.*)"
   res <- regexpr(regex, path, perl = TRUE)
@@ -288,28 +345,58 @@ is_url <- function(path) {
   grepl("^((http|ftp)s?|sftp)://", path)
 }
 
-check_path <- function(path) {
+download_file <- function(url, ext, call = caller_env()) {
+  local_path <- vroom_tempfile(fileext = ext, pattern = "vroom-download-url-")
+  show_progress <- vroom_progress()
+
+  try_fetch(
+    if (requireNamespace("curl", quietly = TRUE)) {
+      curl::curl_download(url, local_path, quiet = !show_progress)
+    } else {
+      utils::download.file(url, local_path, mode = "wb", quiet = !show_progress)
+    },
+    error = function(cnd) {
+      unlink(local_path)
+      cli::cli_abort(
+        c(
+          "Failed to download {.url {url}}.",
+          "x" = conditionMessage(cnd)
+        ),
+        parent = NA,
+        error = cnd,
+        call = call
+      )
+    }
+  )
+
+  local_path
+}
+
+check_path <- function(path, call = caller_env()) {
   if (file.exists(path)) {
     return(normalizePath_utf8(path, mustWork = FALSE))
   }
 
-  stop(
-    "'",
-    path,
-    "' does not exist",
-    if (!is_absolute_path(path)) {
-      paste0(" in current working directory ('", getwd(), "')")
-    },
-    ".",
-    call. = FALSE
+  where <- function(path) {
+    if (is_absolute_path(path)) {
+      ""
+    } else {
+      " in current working directory: {.file {getwd()}}"
+    }
+  }
+  msg <- glue(
+    "{.file {path}} does not exist<<where(path)>>.",
+    .open = "<<",
+    .close = ">>"
   )
+  cli::cli_abort(msg, call = call)
 }
 
 is_absolute_path <- function(path) {
   grepl("^(/|[A-Za-z]:|\\\\|~)", path)
 }
 
-zipfile <- function(path, open = "r") {
+zipfile <- function(path, open = "") {
   files <- utils::unzip(path, list = TRUE)
   file <- files$Name[[1]]
 
@@ -322,13 +409,13 @@ zipfile <- function(path, open = "r") {
 
 utils::globalVariables("con")
 
-chr_to_file <- function(x, envir = parent.frame()) {
-  out <- tempfile()
+chr_to_file <- function(x, call = caller_env()) {
+  out <- vroom_tempfile(pattern = "vroom-chr-to-file-")
   con <- file(out, "wb")
   writeLines(sub("\n$", "", x), con, useBytes = TRUE)
   close(con)
 
-  withr::defer(unlink(out), envir = envir)
+  withr::defer(unlink(out), envir = call)
 
   normalizePath_utf8(out)
 }
