@@ -10,8 +10,6 @@
 cpp11::strings read_chr(vroom_vec_info* info);
 SEXP check_na(SEXP na, SEXP val);
 
-#ifdef HAS_ALTREP
-
 struct vroom_chr : vroom_vec {
 
 public:
@@ -125,7 +123,5 @@ public:
   }
 };
 
-#endif
-
 // Called the package is loaded
 [[cpp11::init]] void init_vroom_chr(DllInfo* dll);
