@@ -9,8 +9,6 @@ double bsd_strtod(const char* begin, const char* end, const char decimalMark);
 
 cpp11::doubles read_dbl(vroom_vec_info* info);
 
-#ifdef HAS_ALTREP
-
 /* Vroom Dbl */
 
 class vroom_dbl : public vroom_vec {
@@ -84,7 +82,7 @@ public:
   }
 
   static void* Dataptr(SEXP vec, Rboolean) {
-    return STDVEC_DATAPTR(Materialize(vec));
+    return DATAPTR_RW(Materialize(vec));
   }
 
   // -------- initialize the altrep class with the methods above
@@ -105,7 +103,5 @@ public:
   }
 };
 
-#endif
-
 // Called the package is loaded
 [[cpp11::init]] void init_vroom_dbl(DllInfo* dll);
