@@ -1,5 +1,6 @@
 #include "grisu3.h"
 #include <array>
+#include <functional>
 #include <future>
 #include <iterator>
 
@@ -253,28 +254,15 @@ template <> void write_buf(const std::vector<char>& buf, SEXP& con) {
   R_WriteConnection(con, (void*)buf.data(), sizeof buf[0] * buf.size());
 }
 
-#ifdef VROOM_USE_CONNECTIONS_API
-void write_buf_con(
-    const std::vector<char>& buf, Rconnection con, bool is_stdout) {
-  if (is_stdout) {
-    std::string out;
-    std::copy(buf.begin(), buf.end(), std::back_inserter(out));
-    Rprintf("%.*s", buf.size(), out.c_str());
-  } else {
-    R_WriteConnection(con, (void*)buf.data(), sizeof buf[0] * buf.size());
-  }
-}
-#else
 void write_buf_con(const std::vector<char>& buf, SEXP con, bool is_stdout) {
   if (is_stdout) {
     std::string out;
     std::copy(buf.begin(), buf.end(), std::back_inserter(out));
-    Rprintf("%.*s", buf.size(), out.c_str());
+    Rprintf("%.*s", (int) buf.size(), out.c_str());
   } else {
     write_buf(buf, con);
   }
 }
-#endif
 
 std::vector<SEXPTYPE> get_types(const cpp11::list& input) {
   std::vector<SEXPTYPE> out;
@@ -381,7 +369,7 @@ void vroom_write_out(
       auto end = begin + num_lines;
       futures[idx][t++] = std::async(
           fill_buf,
-          input,
+          std::cref(input),
           delim,
           eol,
           na_str,
@@ -492,7 +480,7 @@ void vroom_write_out(
 
   bool should_open = !is_open(con);
   if (should_open) {
-    cpp11::package("base")["open"](con, mode);
+    cpp11::package("vroom")["open_safely"](con, mode);
   }
 
   bool should_close = should_open;
@@ -526,7 +514,7 @@ void vroom_write_out(
       auto end = begin + num_lines;
       futures[idx][t++] = std::async(
           fill_buf,
-          input,
+          std::cref(input),
           delim,
           eol,
           na_str,
