@@ -95,8 +95,6 @@ test_that("problems with number of columns works for single files", {
 })
 
 test_that("parsing problems are shown for all datatypes", {
-  skip_if(getRversion() < "3.5")
-
   types <- list(
     "an integer" = col_integer(),
     "a big integer" = col_big_integer(),
@@ -145,10 +143,6 @@ test_that("parsing problems are shown for all datatypes", {
 })
 
 test_that("problems that are generated more than once are not duplicated", {
-  # On versions of R without ALTREP the warnings will happen at different times,
-  # so we skip this test in those cases
-  skip_if(getRversion() < "3.5")
-
   res <- vroom(I("x\n1\n2\n3\n4\n5\na"), col_types = "i", delim = ",")
 
   # generate first problem
@@ -207,16 +201,16 @@ test_that("can promote vroom parse warning to error", {
     x$a[1]
   }
 
-  expect_error(
-    # This fails hard if we unwind protect the warning (aborts RStudio)
-    # - Try to throw error after catching the warning
+  # This fails hard if we unwind protect the warning (aborts RStudio)
+  # - Try to throw error after catching the warning
+  expect_snapshot(error = TRUE, {
     withCallingHandlers(
       expr = make_warning(),
       vroom_parse_issue = function(cnd) {
         abort("oh no")
       }
     )
-  )
+  })
 })
 
 test_that("emits an error message if provided incorrect input", {
@@ -238,7 +232,7 @@ test_that("problems are correct even if print is first encounter", {
     show_col_types = FALSE
   )
 
-  print(foo)
+  expect_output(expect_warning(print(foo), class = "vroom_parse_issue"))
 
   probs <- problems(foo)
   expect_equal(probs$row, c(3, 5))
@@ -252,7 +246,7 @@ test_that("problems are correct even if print is first encounter", {
     show_col_types = FALSE
   )
 
-  print(foo)
+  expect_output(expect_warning(print(foo), class = "vroom_parse_issue"))
 
   probs <- problems(foo)
   expect_equal(probs$row, c(2, 4))
